use clap::{Parser, Subcommand};
use dotenvy::dotenv;
use ethereum_types::Address;
use ethers::prelude::*;
use ethers::{providers::Provider, signers::LocalWallet};
use ethnum::U256;
use tracing::info;

#[derive(Parser)]
#[clap(
    name = "truite-injector-toolbox",
    about = "Toolbox for injecting tx and bx in an Ecomobicoin node.",
    author, version,
    long_about = None
)]
#[command(propagate_version = true)]
struct Opts {
    #[command(subcommand)]
    command: SubCommand,
}

#[derive(Subcommand, Debug)]
enum SubCommand {
    /// Send tx
    Tx {
        to: Address,
        #[clap(long, default_value_t = U256::from(210_000u128))]
        value: U256,
    },
    /// Send bx
    Bx {
        #[clap(long)]
        quantity: Option<U256>,
        #[clap(long)]
        timestamp: u64,
        #[clap(long)]
        data: Option<Bytes>,
    },
    /// Feed a block
    Block {},
}
#[tokio::main]
async fn main() {
    let cli = Opts::parse();

    dotenv().expect(".env file not found");
    // for (key, value) in env::vars() {
    //     println!("{key}: {value}");
    // }

    let signing_key = std::env::var("SIGNING_KEY").unwrap();
    let signer: LocalWallet = signing_key.parse::<LocalWallet>().unwrap();

    let rpc_url: String =
        std::env::var("RPC_URL").unwrap_or_else(|_| "http://127.0.0.1:8545".into());
    let provider = Provider::<Http>::try_from(rpc_url.clone()).unwrap();

    let chain_id: u64 = std::env::var("CHAIN_ID")
        .unwrap_or_else(|_| String::from("63"))
        .parse::<u64>()
        .unwrap();

    let block_number: U64 = provider.get_block_number().await.unwrap_or_else(|_| {
        panic!(
            "Failed to get block number from {:?}, cannot go further",
            rpc_url
        )
    });
    // connect the wallet to the provider
    let client = SignerMiddleware::new(provider, signer.clone().with_chain_id(chain_id));
    info!(
        "Connected to chain id: {:?}, rpc: {:?}, latest block number: {:?}.",
        chain_id, rpc_url, block_number
    );

    match &cli.command {
        SubCommand::Tx { to, value } => {
            println!("tx")
        }
        SubCommand::Bx {
            quantity,
            timestamp,
            data,
        } => println!("bx"),
        SubCommand::Block {} => println!("bloc"),
    }
}
