pub mod txbx_helper {
    use actix_web::{get, post, web, App, HttpResponse, HttpServer, Responder};
    use ecomobicoin_jsonrpc::types::{self, Bytes};
    use ethereum_types::{Address, U64};
    use ethers::{
        prelude::*,
        types::transaction::ecomobicoin_behavior::EcoMobiCoinBehaviorTransactionRequest,
        utils::Anvil,
    };
    use ethnum::U256;
    use std::{env, str::FromStr, time::SystemTime};
    use tracing::info;

    pub fn build_default_tx() -> ecomobicoin_jsonrpc::types::TransactionMessage {
        build_tx(None, None, None, None, None, None, None)
    }

    pub fn build_tx(
        chain_id: Option<U64>,
        nonce: Option<U64>,
        to: Option<Address>,
        gas: Option<U64>,
        gas_price: Option<U256>,
        value: Option<U256>,
        input: Option<Bytes>,
    ) -> ecomobicoin_jsonrpc::types::TransactionMessage {
        let chain_id = chain_id.unwrap_or_else(|| 636363.into());
        let nonce = nonce.unwrap_or_else(|| 1.into());
        // let to = to.unwrap_or_default();
        let gas = gas.unwrap_or_default();
        let gas_price = gas_price.unwrap_or_default();
        let value = value.unwrap_or_default();
        let input = input.unwrap_or_default();

        types::TransactionMessage::EIP2930 {
            chain_id,
            nonce,
            to,
            gas,
            gas_price,
            value,
            input,
            access_list: [].to_vec(),
        }
    }

    pub fn build_default_bx() -> ecomobicoin_jsonrpc::types::BehaviorMessage {
        build_bx(None, None, None, None, None)
    }

    pub fn build_bx(
        chain_id: Option<U64>,
        to: Option<Address>,
        timestamp: Option<U64>,
        quantity: Option<U64>,
        input: Option<Bytes>,
    ) -> types::BehaviorMessage {
        let chain_id = chain_id.unwrap_or_else(|| 636363.into());
        let now = SystemTime::now()
            .duration_since(SystemTime::UNIX_EPOCH)
            .unwrap()
            .as_secs();
        let timestamp = timestamp.unwrap_or(now.into());
        let quantity = quantity.unwrap_or(10_000.into());
        let input = input.unwrap_or_default();

        types::BehaviorMessage {
            chain_id,
            to,
            timestamp,
            quantity,
            input,
        }
    }
}

#[cfg(test)]
mod tests {
    use std::default;

    use super::*;
    use crate::res::chainspec::MAINNET;

    #[test]
    fn build_default_tx_test() {
        // ARRANGE
        // ACT
        let default_tx = build_default_tx();
        // ASSERT
        // default_tx.
    }

    #[test]
    fn build_default_bx_test() {}
}
